#include "colorTracker.h"


//--------------------------------------------------------------
void colorTracker::setup(int _w, int _h){
	
	grayPixels = new unsigned char[_w*_h];
	grayImage.allocate(_w, _h);
	w = _w;
	h = _h;
	
	hueRange = 20;
	satRange = 30;
	valRange = 25;
	
	
	colorTrackerPix = new unsigned char [64*64*3];
	
	setHueSatBri(0, 0, 0);
}

//--------------------------------------------------------------
void colorTracker::setHueSatBri(int _hue, int _sat, int _bri){
	
	hue = _hue;
	sat = _sat;
	val = _bri;
	
	myColor = convertHsvToRgb(ofPoint(hue * (255.0f/180.0f), sat, val));
	
	updateColorImage();

}

//--------------------------------------------------------------
void colorTracker::updateColorImage(){
	// do some maths here:
	for (int i = 0;  i < 64; i++){
		for (int j = 0; j < 64; j++){
			
			float pctw = (float) i / 63.0f;
			float pcth = (float) j / 63.0f;
			
			int h = (hue - hueRange) + (float)hueRange * 2 * pctw;
			int s = (sat - satRange) + (float)satRange * 2 * pcth;
			if (h < 0) h += 180;
			if (h > 180) h -= 180;
			if (s < 0) s = 0;
			if (s > 255) s = 255;
			
			
			colorTrackerPix[ (j * 64 + i) * 3] = h;
			colorTrackerPix[ (j * 64 + i) * 3 + 1] = s;
			colorTrackerPix[ (j * 64 + i) * 3 + 2] = val;
			
		}
	}
	
	colorTrackerImg.setFromHsvPixels(colorTrackerPix);
	
	myColor = convertHsvToRgb(ofPoint(hue * (255.0f/180.0f), sat, val));

}

//--------------------------------------------------------------
void colorTracker::trackColors(unsigned char * colorHsvPixels){

	// to do make this not suck with hues, like 0 (vs 255, for example).
	
	for (int i = 0; i < w*h; i++){
		
		
		if ((colorHsvPixels[i*3] > (hue - hueRange) && colorHsvPixels[i*3] < (hue + hueRange)) &&
			(colorHsvPixels[i*3+1] > (sat - satRange) && colorHsvPixels[i*3+1] < (sat + satRange)) &&
			(colorHsvPixels[i*3+2] > (val - valRange) && colorHsvPixels[i*3+2] < (val + valRange))){
			
			grayPixels[i] = 255;
			
			
		} else {
			
			grayPixels[i] = 0;
		}
		
	}
	
	grayImage.setFromPixels(grayPixels, w, h);
	
	for (int i = 0; i < nDilations; i++){
		grayImage.dilate();
	}
	
	// first, try nearby: 
	float xToTry1 = MAX(trackedPoint.x - 80, 0);
	float yToTry1 = MAX(trackedPoint.y - 80, 0);
	float xToTry2 = MIN(trackedPoint.x + 80, grayImage.width);
	float yToTry2 = MIN(trackedPoint.y + 80, grayImage.height);

	grayImage.setROI(ofRectangle(xToTry1,yToTry1, xToTry2-xToTry1, yToTry2-yToTry1));
	contourFinder.findContours(grayImage, 20, (340*240)/3, 1, false);	
	grayImage.resetROI();
	
	if (contourFinder.nBlobs > 0){
		trackedPoint.x = contourFinder.blobs[0].centroid.x + xToTry1;
		trackedPoint.y = contourFinder.blobs[0].centroid.y + yToTry1;
	} else {
		//try again, now with abit more: 
		
		float xToTry1 = MAX(trackedPoint.x - 160, 0);
		float yToTry1 = MAX(trackedPoint.y - 160, 0);
		float xToTry2 = MIN(trackedPoint.x + 160, grayImage.width);
		float yToTry2 = MIN(trackedPoint.y + 160, grayImage.height);
		
		
		grayImage.setROI(ofRectangle(xToTry1,yToTry1, xToTry2-xToTry1, yToTry2-yToTry1));
		contourFinder.findContours(grayImage, 20, (340*240)/3, 1, false);	
		grayImage.resetROI();
				
		if (contourFinder.nBlobs > 0){
			trackedPoint.x = contourFinder.blobs[0].centroid.x + xToTry1;
			trackedPoint.y = contourFinder.blobs[0].centroid.y + yToTry1;
		} else {
			
			contourFinder.findContours(grayImage, 20, (340*240)/3, 1, false);	
		
			if (contourFinder.nBlobs > 0){
				trackedPoint.x = contourFinder.blobs[0].centroid.x;
				trackedPoint.y = contourFinder.blobs[0].centroid.y;
			} 
				
			
		}
		
	}
	
	
	
}
		