#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){

	
	
	
	xmlSettings.loadFile("settings.xml");
	
	bUseVideo = xmlSettings.getValue("use_video_file", true);
	
	if (bUseVideo == true){
		string fileName = xmlSettings.getValue("video_file_name", "new_track-1.mov");
		vidPlayer.loadMovie(fileName);
		vidPlayer.play();	
		w = vidPlayer.width;
		h = vidPlayer.height;
	} else {
		ofSetLogLevel(OF_LOG_NOTICE);
		printf("------------------------ \n");
		vidGrabber.listDevices();
		printf("------------------------ \n");
		ofSetLogLevel(OF_LOG_ERROR);
		
		int id = xmlSettings.getValue("video_grabber:id", 0);
		int width = xmlSettings.getValue("video_grabber:width", 0);
		int height = xmlSettings.getValue("video_grabber:height", 0);
		if (id != 0){
			vidGrabber.setDeviceID(id);
		}
		vidGrabber.initGrabber(width, height);
		w = vidGrabber.width;
		h = vidGrabber.height;
	}

	bEverGotColorPixels = false;

	fileNamePrefix =  xmlSettings.getValue("output_name_prefix", "image_");
	outputCount = 0;
	
    colorImg.allocate(w,h);
	
	for (int i = 0; i < 4; i++){
		CT[i].setup(w,h);
	}
	
	bClicked = false;
	
	for (int i = 0; i < 4; i++){
		offscreenTexture[i].allocate(w,h,false);
		offscreenTexture[i].clear();
	}
	
	bOutputImage = false;
	bJustStartedTracking = true;
	

	//ofPoint				smoothedPoint[4];
	//bool					bJustStartedTracking;
	
	
	//--------------------------------------------------------------------	panel
	panel.setup("control", 750,0,300,700);
	
	panel.addPanel("control", 1, false);
	panel.addPanel("color selection", 1, false);
	panel.addPanel("color (a)",1, false);
	panel.addPanel("color (b)",1, false);
	panel.addPanel("color (c)",1, false);
	panel.addPanel("color (d)",1, false);

	panel.setWhichPanel("control");
	panel.setWhichColumn(0);
	panel.addToggle("tracking", "TRACKING", false); 
	panel.addToggle("pause video file", "PAUSE_VIDEO", false); 
	panel.addToggle("draw threshold", "DRAW_THRESH", false); 
	panel.addToggle("draw lines", "DRAW_LINES", false); 
	panel.addToggle("clear lines", "TOGGLE_LINE_CLEAR", false); 
	panel.addToggle("save lines (to jpg)", "TOGGLE_LINE_SAVE", false); 
	panel.addToggle("show video settings", "TOGGLE_VID_SETTINGS", false); 
	
	panel.addSlider("smoothing", "SMOOTHING", 0, 0, 1,  false);
	
	
	panel.setWhichPanel("color selection");
	panel.setWhichColumn(0);
	
	vector <string> boxNames;
	boxNames.push_back("color a");
	boxNames.push_back("color b");
	boxNames.push_back("color c");
	boxNames.push_back("color d");
	panel.addMultiToggle("color selection on click", "COLOR_SELECT", 0, boxNames);
	
	for (int i = 0; i < 4; i++){
		panel.addDrawableRect("", &CT[i].colorTrackerImg, 64, 64);
	}
	
	for (int i = 0; i < 4; i++){
		panel.setWhichPanel(2 + i);
		panel.addDrawableRect("", &CT[i].colorTrackerImg, 64, 64);
		
		
		panel.addSlider("hue", "COLOR_" + ofToString(i) + "_HUE", 0, 0, 180,  true);
		panel.addSlider("saturation", "COLOR_" + ofToString(i) + "_SAT", 0, 0, 255,  true);
		panel.addSlider("value", "COLOR_" + ofToString(i) + "_VAL", 0, 0, 255,  true);
		
		panel.addSlider("hue range", "COLOR_" + ofToString(i) + "_HUE_RANGE", 10, 0, 127,  true);
		panel.addSlider("sat range", "COLOR_" + ofToString(i) + "_SAT_RANGE", 10, 0, 127,  true);
		panel.addSlider("val range", "COLOR_" + ofToString(i) + "_VAL_RANGE", 30, 0, 127,  true);
		
		panel.addSlider("num dilations", "COLOR_" + ofToString(i) + "_N_DILATIONS", 0, 0, 10,  true);
	}
	
	panel.loadSettings("controlPanel.xml");
	//--------------------------------------------------------------------	
	
}

//--------------------------------------------------------------
void testApp::update(){
	
	
	
	
	//--------------------------------------------------------------------	panel
	int whichColor		= panel.getValueI("COLOR_SELECT", 0);
	int bPaused			= panel.getValueB("PAUSE_VIDEO", 0);
	bool bDrawThresh	= panel.getValueB("DRAW_THRESH", 0);
	bool bSettings		= panel.getValueB("TOGGLE_VID_SETTINGS",0);
	bool bTracking		= panel.getValueB("TRACKING"); 
	float smoothing		= panel.getValueF("SMOOTHING");
	
	if (!bTracking){
		bJustStartedTracking = true;
	}
	
	if (bSettings == true){
		panel.setValueB("TOGGLE_VID_SETTINGS", false,0);
		if (!bUseVideo){
			vidGrabber.videoSettings();
		}
	}
	
	
	bool bClearLines	= panel.getValueB("TOGGLE_LINE_CLEAR",0);
	if (bClearLines == true){
		for (int i = 0; i < 4; i++){
			offscreenTexture[i].swapIn();
			offscreenTexture[i].clear();
			offscreenTexture[i].swapOut();
		}	
		for (int i = 0; i < 4; i++){
			pts[i].clear();	
		}
		panel.setValueB("TOGGLE_LINE_CLEAR",false,0);
	}
	
		
		
	
	bool bChanged[4];
	
	for (int i = 0; i < 4; i++){
		
		bChanged[i] = false;
		
		int prevHue = CT[i].hue;
		int prevSat = CT[i].sat;
		int prevVal = CT[i].val;
		int prevHueRange = CT[i].hueRange;
		int prevSatRange = CT[i].satRange;
		int prevValRange = CT[i].valRange;
		
		int prevDilations = CT[i].nDilations;


		CT[i].hueRange			= panel.getValueI("COLOR_" + ofToString(i) + "_HUE_RANGE", 0);
		CT[i].satRange			= panel.getValueI("COLOR_" + ofToString(i) + "_SAT_RANGE", 0);
		CT[i].valRange			= panel.getValueI("COLOR_" + ofToString(i) + "_VAL_RANGE", 0);
		
		CT[i].nDilations			= panel.getValueI("COLOR_" + ofToString(i) + "_N_DILATIONS", 0);
	
		CT[i].hue				= panel.getValueI("COLOR_" + ofToString(i) + "_HUE", 0);
		CT[i].sat				= panel.getValueI("COLOR_" + ofToString(i) + "_SAT", 0);
		CT[i].val				= panel.getValueI("COLOR_" + ofToString(i) + "_VAL", 0);
		
		if (prevHue != CT[i].hue) bChanged[i] = true;
		if (prevSat != CT[i].sat) bChanged[i] = true;
		if (prevVal != CT[i].val) bChanged[i] = true;
		if (prevHueRange != CT[i].hueRange) bChanged[i] = true;
		if (prevSatRange != CT[i].satRange) bChanged[i] = true;
		if (prevValRange != CT[i].valRange) bChanged[i] = true;
		if (prevDilations != CT[i].nDilations) bChanged[i] = true;
		
		CT[i].updateColorImage();		// [todo make this not every frame]
	}
	
	panel.update();
	
	int whichPanel = panel.selectedPanel;
	//--------------------------------------------------------------------	
	
	if (bUseVideo == true){
		if (bPaused == true){
			vidPlayer.setPaused(true);	
		}  else {
			vidPlayer.setPaused(false);
		}
	} 
	
	ofBackground(0,0,0);

    bool bNewFrame = false;
	
	unsigned char * pixels;
	
	if (bUseVideo == true){
		vidPlayer.idleMovie();
		bNewFrame = vidPlayer.isFrameNew();
		pixels = vidPlayer.getPixels();
	} else {
		vidGrabber.grabFrame(); 
		bNewFrame = vidGrabber.isFrameNew();
		pixels = vidGrabber.getPixels();
	}
	
	if (bNewFrame == true){
		colorImg.setFromPixels(pixels, w,h);
		colorImg.convertRgbToHsv();
		colorPixels = colorImg.getPixels();
		bEverGotColorPixels = true;
	}
	
	
	if (bEverGotColorPixels && bTracking)
	for (int i = 0; i < 4; i++){
		if (bNewFrame == true || bChanged[i]){
			CT[i].trackColors(colorPixels);
			if (CT[i].contourFinder.nBlobs == 1){
				
				
				if (bJustStartedTracking == true){
					smoothedPoint[i].x = CT[i].trackedPoint.x;
					smoothedPoint[i].y = CT[i].trackedPoint.y;
					bJustStartedTracking = false;
				} else {
					smoothedPoint[i].x = (smoothing) * smoothedPoint[i].x + 
										 (1-smoothing) * CT[i].trackedPoint.x;
					smoothedPoint[i].y = (smoothing) * smoothedPoint[i].y + 
										 (1-smoothing) * CT[i].trackedPoint.y;
				}
				pts[i].push_back(ofPoint(smoothedPoint[i].x, smoothedPoint[i].y, 0));
				
				//pts[i].push_back(ofPoint(CT[i].trackedPoint.x, CT[i].trackedPoint.y, 0));
			}
			
		}
	}
	
	
	
	

	
	//------------------------------------------------------------------ handle click
	if (bClicked == true){
		if (whichPanel == 1){
			unsigned char * colorPixels = colorImg.getPixels();
			if (mouseX >= 0 && mouseX < colorImg.width){
				if (mouseY >= 0 && mouseY < colorImg.height){
					
					int position = (mouseY * colorImg.width + mouseX) * 3;
					int hue = colorPixels[position];
					int sat = colorPixels[position+1];
					int val = colorPixels[position+2];
					
					panel.setValueI( "COLOR_" + ofToString(whichColor) + "_HUE", hue, 0);
					panel.setValueI( "COLOR_" + ofToString(whichColor) + "_SAT", sat, 0);
					panel.setValueI( "COLOR_" + ofToString(whichColor) + "_VAL", val, 0);
					CT[whichColor].setHueSatBri(hue, sat, val);
					CT[whichColor].trackedPoint.x = mouseX;
					CT[whichColor].trackedPoint.y = mouseY;
				}
			}
		}
		
		bClicked = false;
	}
	//------------------------------------------------------------------ 
	
}

//--------------------------------------------------------------
void testApp::draw(){
	
	int whichColor		= panel.getValueI("COLOR_SELECT", 0);
	int bPaused			= panel.getValueB("PAUSE_VIDEO", 0);
	bool bDrawThresh	= panel.getValueB("DRAW_THRESH", 0);
	bool bDrawLines		= panel.getValueB("DRAW_LINES", 0);
	int whichPanel		= panel.selectedPanel;
	
	
	bool bSaveLines =  panel.getValueB("TOGGLE_LINE_SAVE",0);
	if (bSaveLines == true){
		bOutputImage = true;
	}
	panel.setValueB("TOGGLE_LINE_SAVE",false,0);
	
	
	if (bOutputImage == true){
		ofSetColor(255, 255, 255);
		ofRect(0,0,w,h);
		
		if (bDrawLines == true){
			ofEnableAlphaBlending();
			glBlendFunc(GL_DST_COLOR, GL_ZERO);
			ofSetColor(255, 255, 255, 255);
			for (int i = 0; i < 4; i++){
				offscreenTexture[i].draw(0,0,w,h);
			}
			ofEnableAlphaBlending();
		}

		ofImage img;
		img.grabScreen(0, 0, w, h);
		
		string fileName = "output/" + fileNamePrefix + ofToString(outputCount) + ".png";
		img.saveImage(fileName);
		bOutputImage = false;
		outputCount++;
		return;
	}
	
	
	

	
	
	// draw the incoming, the grayscale, the bg and the thresholded difference
	ofSetColor(0xffffff);
	
	if (bUseVideo){
		vidPlayer.draw(0,0);
	} else {
		vidGrabber.draw(0,0);	
	}
		
	if (bDrawThresh == true){
		if (whichPanel == 0 || whichPanel == 1){
			CT[whichColor].grayImage.draw(0,0);
		} else {
			CT[whichPanel-2].grayImage.draw(0,0);
		}
	}
	
	//CT.contourFinder.draw(0,0);
	
	
	ofEnableAlphaBlending();
	ofEnableSmoothing();
	
	ofSetColor(255,0,255,180);
	
	
	if (bDrawLines == true){
		
	
		for (int i = 0; i < 4; i++){
			offscreenTexture[i].swapIn();
			offscreenTexture[i].setupScreenForMe();
						
			ofEnableSmoothing();
			ofEnableAlphaBlending();
			ofSetLineWidth(4);
			ofSetColor(CT[i].myColor.x, CT[i].myColor.y, CT[i].myColor.z, 255);
			if (pts[i].size() > 1){
				
				int size = pts[i].size()-1;
				int size2 = pts[i].size()-2;
				ofLine(pts[i][size].x, pts[i][size].y, pts[i][size2].x, pts[i][size2].y);
				
			}
			ofSetLineWidth(1);
			
			
			offscreenTexture[i].setupScreenForThem();
			offscreenTexture[i].swapOut();
		}
		
	}  else  {
		
	}
	
	
	if (bDrawLines == true){
		ofEnableAlphaBlending();
		glBlendFunc(GL_DST_COLOR, GL_ZERO);
		ofSetColor(255, 255, 255, 255);
		for (int i = 0; i < 4; i++){
			offscreenTexture[i].draw(0,0,w,h);
		}
		ofEnableAlphaBlending();
	}

	
	
	
	
	
	for (int i = 0; i < 4; i++){
		ofSetColor(CT[i].myColor.x, CT[i].myColor.y, CT[i].myColor.z, 100);
		ofCircle(CT[i].trackedPoint.x, CT[i].trackedPoint.y, 10);
	}
	
	
	panel.draw();
	
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){


	switch (key){
		case 's':
			panel.setValueB("TRACKING", !panel.getValueB("TRACKING"));
			break;
		case 'd':
			panel.setValueB("DRAW_THRESH", !panel.getValueB("DRAW_THRESH"));
			break;
		case ' ':
			
			panel.setValueB("TRACKING", !panel.getValueB("TRACKING"));
			if (panel.getValueB("TRACKING")){
				panel.setValueB("TOGGLE_LINE_CLEAR", true);
			}
			break;
		
		case 'o':
			panel.setValueB("TOGGLE_LINE_SAVE", true);
			break;
			
	}
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	panel.mouseDragged(x,y,button);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	panel.mousePressed(x,y,button);
	bClicked = true;
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	panel.mouseReleased();
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

